@@ -9,7 +9,9 @@ The View:
   removed from the store.
 """
 import collections
-import typing
+import re
+from collections.abc import Iterator, MutableMapping, Sequence
+from typing import Any, Optional
 
 import blinker
 import sortedcontainers
@@ -17,6 +19,7 @@ import sortedcontainers
 import mitmproxy.flow
 from mitmproxy import command
 from mitmproxy import ctx
+from mitmproxy import dns
 from mitmproxy import exceptions
 from mitmproxy import hooks
 from mitmproxy import connection
@@ -42,7 +45,7 @@ class _OrderKey:
     def __init__(self, view):
         self.view = view
 
-    def generate(self, f: mitmproxy.flow.Flow) -> typing.Any:  # pragma: no cover
+    def generate(self, f: mitmproxy.flow.Flow) -> Any:  # pragma: no cover
         pass
 
     def refresh(self, f):
@@ -73,7 +76,7 @@ class _OrderKey:
 
 class OrderRequestStart(_OrderKey):
     def generate(self, f: mitmproxy.flow.Flow) -> float:
-        return f.timestamp_start
+        return f.timestamp_created
 
 
 class OrderRequestMethod(_OrderKey):
@@ -82,6 +85,8 @@ class OrderRequestMethod(_OrderKey):
             return f.request.method
         elif isinstance(f, tcp.TCPFlow):
             return "TCP"
+        elif isinstance(f, dns.DNSFlow):
+            return dns.op_codes.to_str(f.request.op_code)
         else:
             raise NotImplementedError()
 
@@ -92,6 +97,8 @@ class OrderRequestURL(_OrderKey):
             return f.request.url
         elif isinstance(f, tcp.TCPFlow):
             return human.format_address(f.server_conn.address)
+        elif isinstance(f, dns.DNSFlow):
+            return f.request.questions[0].name if f.request.questions else ""
         else:
             raise NotImplementedError()
 
@@ -110,12 +117,12 @@ class OrderKeySize(_OrderKey):
             for message in f.messages:
                 size += len(message.content)
             return size
+        elif isinstance(f, dns.DNSFlow):
+            return f.response.size if f.response else 0
         else:
             raise NotImplementedError()
 
 
-matchall = flowfilter.parse("~http | ~tcp")
-
 orders = [
     ("t", "time"),
     ("m", "method"),
@@ -128,22 +135,22 @@ class View(collections.abc.Sequence):
     def __init__(self):
         super().__init__()
         self._store = collections.OrderedDict()
-        self.filter = matchall
+        self.filter = flowfilter.match_all
         # Should we show only marked flows?
         self.show_marked = False
 
         self.default_order = OrderRequestStart(self)
         self.orders = dict(
-            time=OrderRequestStart(self), method=OrderRequestMethod(self),
-            url=OrderRequestURL(self), size=OrderKeySize(self),
+            time=OrderRequestStart(self),
+            method=OrderRequestMethod(self),
+            url=OrderRequestURL(self),
+            size=OrderKeySize(self),
         )
         self.order_key = self.default_order
         self.order_reversed = False
         self.focus_follow = False
 
-        self._view = sortedcontainers.SortedListWithKey(
-            key=self.order_key
-        )
+        self._view = sortedcontainers.SortedListWithKey(key=self.order_key)
 
         # The sig_view* signals broadcast events that affect the view. That is,
         # an update to a flow in the store but not in the view does not trigger
@@ -167,21 +174,20 @@ class View(collections.abc.Sequence):
 
     def load(self, loader):
         loader.add_option(
-            "view_filter", typing.Optional[str], None,
-            "Limit the view to matching flows."
+            "view_filter", Optional[str], None, "Limit the view to matching flows."
         )
         loader.add_option(
-            "view_order", str, "time",
+            "view_order",
+            str,
+            "time",
             "Flow sort order.",
             choices=list(map(lambda c: c[1], orders)),
         )
         loader.add_option(
-            "view_order_reversed", bool, False,
-            "Reverse the sorting order."
+            "view_order_reversed", bool, False, "Reverse the sorting order."
         )
         loader.add_option(
-            "console_focus_follow", bool, False,
-            "Focus follows new flows."
+            "console_focus_follow", bool, False, "Focus follows new flows."
         )
 
     def store_count(self):
@@ -189,7 +195,7 @@ class View(collections.abc.Sequence):
 
     def _rev(self, idx: int) -> int:
         """
-            Reverses an index, if needed
+        Reverses an index, if needed
         """
         if self.order_reversed:
             if idx < 0:
@@ -203,7 +209,7 @@ class View(collections.abc.Sequence):
     def __len__(self):
         return len(self._view)
 
-    def __getitem__(self, offset) -> typing.Any:
+    def __getitem__(self, offset) -> Any:
         return self._view[self._rev(offset)]
 
     # Reflect some methods to the efficient underlying implementation
@@ -212,10 +218,12 @@ class View(collections.abc.Sequence):
         v = self._view.bisect_right(f)
         return self._rev(v - 1) + 1
 
-    def index(self, f: mitmproxy.flow.Flow, start: int = 0, stop: typing.Optional[int] = None) -> int:
+    def index(
+        self, f: mitmproxy.flow.Flow, start: int = 0, stop: Optional[int] = None
+    ) -> int:
         return self._rev(self._view.index(f, start, stop))
 
-    def __contains__(self, f: typing.Any) -> bool:
+    def __contains__(self, f: Any) -> bool:
         return self._view.__contains__(f)
 
     def _order_key_name(self):
@@ -240,9 +248,9 @@ class View(collections.abc.Sequence):
     @command.command("view.focus.go")
     def go(self, offset: int) -> None:
         """
-            Go to a specified offset. Positive offests are from the beginning of
-            the view, negative from the end of the view, so that 0 is the first
-            flow, -1 is the last flow.
+        Go to a specified offset. Positive offests are from the beginning of
+        the view, negative from the end of the view, so that 0 is the first
+        flow, -1 is the last flow.
         """
         if len(self) == 0:
             return
@@ -257,7 +265,7 @@ class View(collections.abc.Sequence):
     @command.command("view.focus.next")
     def focus_next(self) -> None:
         """
-            Set focus to the next flow.
+        Set focus to the next flow.
         """
         if self.focus.index is not None:
             idx = self.focus.index + 1
@@ -269,7 +277,7 @@ class View(collections.abc.Sequence):
     @command.command("view.focus.prev")
     def focus_prev(self) -> None:
         """
-            Set focus to the previous flow.
+        Set focus to the previous flow.
         """
         if self.focus.index is not None:
             idx = self.focus.index - 1
@@ -280,9 +288,9 @@ class View(collections.abc.Sequence):
 
     # Order
     @command.command("view.order.options")
-    def order_options(self) -> typing.Sequence[str]:
+    def order_options(self) -> Sequence[str]:
         """
-            Choices supported by the view_order option.
+        Choices supported by the view_order option.
         """
         return list(sorted(self.orders.keys()))
 
@@ -294,12 +302,10 @@ class View(collections.abc.Sequence):
     @command.command("view.order.set")
     def set_order(self, order_key: str) -> None:
         """
-            Sets the current view order.
+        Sets the current view order.
         """
         if order_key not in self.orders:
-            raise exceptions.CommandError(
-                "Unknown flow order: %s" % order_key
-            )
+            raise exceptions.CommandError("Unknown flow order: %s" % order_key)
         order_key = self.orders[order_key]
         self.order_key = order_key
         newview = sortedcontainers.SortedListWithKey(key=order_key)
@@ -321,26 +327,25 @@ class View(collections.abc.Sequence):
     @command.command("view.filter.set")
     def set_filter_cmd(self, filter_expr: str) -> None:
         """
-            Sets the current view filter.
+        Sets the current view filter.
         """
         filt = None
         if filter_expr:
-            filt = flowfilter.parse(filter_expr)
-            if not filt:
-                raise exceptions.CommandError(
-                    "Invalid interception filter: %s" % filter_expr
-                )
+            try:
+                filt = flowfilter.parse(filter_expr)
+            except ValueError as e:
+                raise exceptions.CommandError(str(e)) from e
         self.set_filter(filt)
 
-    def set_filter(self, flt: typing.Optional[flowfilter.TFilter]):
-        self.filter = flt or matchall
+    def set_filter(self, flt: Optional[flowfilter.TFilter]):
+        self.filter = flt or flowfilter.match_all
         self._refilter()
 
     # View Updates
     @command.command("view.clear")
     def clear(self) -> None:
         """
-            Clears both the store and view.
+        Clears both the store and view.
         """
         self._store.clear()
         self._view.clear()
@@ -350,7 +355,7 @@ class View(collections.abc.Sequence):
     @command.command("view.clear_unmarked")
     def clear_not_marked(self) -> None:
         """
-            Clears only the unmarked flows.
+        Clears only the unmarked flows.
         """
         for flow in self._store.copy().values():
             if not flow.marked:
@@ -363,19 +368,15 @@ class View(collections.abc.Sequence):
     @command.command("view.settings.getval")
     def getvalue(self, flow: mitmproxy.flow.Flow, key: str, default: str) -> str:
         """
-            Get a value from the settings store for the specified flow.
+        Get a value from the settings store for the specified flow.
         """
         return self.settings[flow].get(key, default)
 
     @command.command("view.settings.setval.toggle")
-    def setvalue_toggle(
-        self,
-        flows: typing.Sequence[mitmproxy.flow.Flow],
-        key: str
-    ) -> None:
+    def setvalue_toggle(self, flows: Sequence[mitmproxy.flow.Flow], key: str) -> None:
         """
-            Toggle a boolean value in the settings store, setting the value to
-            the string "true" or "false".
+        Toggle a boolean value in the settings store, setting the value to
+        the string "true" or "false".
         """
         updated = []
         for f in flows:
@@ -386,12 +387,10 @@ class View(collections.abc.Sequence):
 
     @command.command("view.settings.setval")
     def setvalue(
-        self,
-        flows: typing.Sequence[mitmproxy.flow.Flow],
-        key: str, value: str
+        self, flows: Sequence[mitmproxy.flow.Flow], key: str, value: str
     ) -> None:
         """
-            Set a value in the settings store for the specified flows.
+        Set a value in the settings store for the specified flows.
         """
         updated = []
         for f in flows:
@@ -401,10 +400,10 @@ class View(collections.abc.Sequence):
 
     # Flows
     @command.command("view.flows.duplicate")
-    def duplicate(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
+    def duplicate(self, flows: Sequence[mitmproxy.flow.Flow]) -> None:
         """
-            Duplicates the specified flows, and sets the focus to the first
-            duplicate.
+        Duplicates the specified flows, and sets the focus to the first
+        duplicate.
         """
         dups = [f.copy() for f in flows]
         if dups:
@@ -413,9 +412,9 @@ class View(collections.abc.Sequence):
             ctx.log.alert("Duplicated %s flows" % len(dups))
 
     @command.command("view.flows.remove")
-    def remove(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
+    def remove(self, flows: Sequence[mitmproxy.flow.Flow]) -> None:
         """
-            Removes the flow from the underlying store and the view.
+        Removes the flow from the underlying store and the view.
         """
         for f in flows:
             if f.id in self._store:
@@ -433,9 +432,9 @@ class View(collections.abc.Sequence):
             ctx.log.alert("Removed %s flows" % len(flows))
 
     @command.command("view.flows.resolve")
-    def resolve(self, flow_spec: str) -> typing.Sequence[mitmproxy.flow.Flow]:
+    def resolve(self, flow_spec: str) -> Sequence[mitmproxy.flow.Flow]:
         """
-            Resolve a flow list specification to an actual list of flows.
+        Resolve a flow list specification to an actual list of flows.
         """
         if flow_spec == "@all":
             return [i for i in self._store.values()]
@@ -449,10 +448,14 @@ class View(collections.abc.Sequence):
             return [i for i in self._store.values() if i.marked]
         elif flow_spec == "@unmarked":
             return [i for i in self._store.values() if not i.marked]
+        elif re.match(r"@[0-9a-f\-,]{36,}", flow_spec):
+            ids = flow_spec[1:].split(",")
+            return [i for i in self._store.values() if i.id in ids]
         else:
-            filt = flowfilter.parse(flow_spec)
-            if not filt:
-                raise exceptions.CommandError("Invalid flow filter: %s" % flow_spec)
+            try:
+                filt = flowfilter.parse(flow_spec)
+            except ValueError as e:
+                raise exceptions.CommandError(str(e)) from e
             return [i for i in self._store.values() if filt(i)]
 
     @command.command("view.flows.create")
@@ -473,7 +476,7 @@ class View(collections.abc.Sequence):
     @command.command("view.flows.load")
     def load_file(self, path: mitmproxy.types.Path) -> None:
         """
-            Load flows into the view, without processing them with addons.
+        Load flows into the view, without processing them with addons.
         """
         try:
             with open(path, "rb") as f:
@@ -487,10 +490,10 @@ class View(collections.abc.Sequence):
         except exceptions.FlowReadException as e:
             ctx.log.error(str(e))
 
-    def add(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
+    def add(self, flows: Sequence[mitmproxy.flow.Flow]) -> None:
         """
-            Adds a flow to the state. If the flow already exists, it is
-            ignored.
+        Adds a flow to the state. If the flow already exists, it is
+        ignored.
         """
         for f in flows:
             if f.id not in self._store:
@@ -501,10 +504,10 @@ class View(collections.abc.Sequence):
                         self.focus.flow = f
                     self.sig_view_add.send(self, flow=f)
 
-    def get_by_id(self, flow_id: str) -> typing.Optional[mitmproxy.flow.Flow]:
+    def get_by_id(self, flow_id: str) -> Optional[mitmproxy.flow.Flow]:
         """
-            Get flow with the given id from the store.
-            Returns None if the flow is not found.
+        Get flow with the given id from the store.
+        Returns None if the flow is not found.
         """
         return self._store.get(flow_id)
 
@@ -512,21 +515,21 @@ class View(collections.abc.Sequence):
     @command.command("view.properties.length")
     def get_length(self) -> int:
         """
-            Returns view length.
+        Returns view length.
         """
         return len(self)
 
     @command.command("view.properties.marked")
     def get_marked(self) -> bool:
         """
-            Returns true if view is in marked mode.
+        Returns true if view is in marked mode.
         """
         return self.show_marked
 
     @command.command("view.properties.marked.toggle")
     def toggle_marked(self) -> None:
         """
-            Toggle whether to show marked views only.
+        Toggle whether to show marked views only.
         """
         self.show_marked = not self.show_marked
         self._refilter()
@@ -534,7 +537,7 @@ class View(collections.abc.Sequence):
     @command.command("view.properties.inbounds")
     def inbounds(self, index: int) -> bool:
         """
-            Is this 0 <= index < len(self)?
+        Is this 0 <= index < len(self)?
         """
         return 0 <= index < len(self)
 
@@ -543,11 +546,10 @@ class View(collections.abc.Sequence):
         if "view_filter" in updated:
             filt = None
             if ctx.options.view_filter:
-                filt = flowfilter.parse(ctx.options.view_filter)
-                if not filt:
-                    raise exceptions.OptionsError(
-                        "Invalid interception filter: %s" % ctx.options.view_filter
-                    )
+                try:
+                    filt = flowfilter.parse(ctx.options.view_filter)
+                except ValueError as e:
+                    raise exceptions.OptionsError(str(e)) from e
             self.set_filter(filt)
         if "view_order" in updated:
             if ctx.options.view_order not in self.orders:
@@ -560,7 +562,7 @@ class View(collections.abc.Sequence):
         if "console_focus_follow" in updated:
             self.focus_follow = ctx.options.console_focus_follow
 
-    def request(self, f):
+    def requestheaders(self, f):
         self.add([f])
 
     def error(self, f):
@@ -590,9 +592,18 @@ class View(collections.abc.Sequence):
     def tcp_end(self, f):
         self.update([f])
 
-    def update(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
+    def dns_request(self, f):
+        self.add([f])
+
+    def dns_response(self, f):
+        self.update([f])
+
+    def dns_error(self, f):
+        self.update([f])
+
+    def update(self, flows: Sequence[mitmproxy.flow.Flow]) -> None:
         """
-            Updates a list of flows. If flow is not in the state, it's ignored.
+        Updates a list of flows. If flow is not in the state, it's ignored.
         """
         for f in flows:
             if f.id in self._store:
@@ -621,12 +632,12 @@ class View(collections.abc.Sequence):
 
 class Focus:
     """
-        Tracks a focus element within a View.
+    Tracks a focus element within a View.
     """
 
     def __init__(self, v: View) -> None:
         self.view = v
-        self._flow: typing.Optional[mitmproxy.flow.Flow] = None
+        self._flow: Optional[mitmproxy.flow.Flow] = None
         self.sig_change = blinker.Signal()
         if len(self.view):
             self.flow = self.view[0]
@@ -635,18 +646,18 @@ class Focus:
         v.sig_view_refresh.connect(self._sig_view_refresh)
 
     @property
-    def flow(self) -> typing.Optional[mitmproxy.flow.Flow]:
+    def flow(self) -> Optional[mitmproxy.flow.Flow]:
         return self._flow
 
     @flow.setter
-    def flow(self, f: typing.Optional[mitmproxy.flow.Flow]):
+    def flow(self, f: Optional[mitmproxy.flow.Flow]):
         if f is not None and f not in self.view:
             raise ValueError("Attempt to set focus to flow not in view")
         self._flow = f
         self.sig_change.send(self)
 
     @property
-    def index(self) -> typing.Optional[int]:
+    def index(self) -> Optional[int]:
         if self.flow:
             return self.view.index(self.flow)
         return None
@@ -683,11 +694,11 @@ class Focus:
 class Settings(collections.abc.Mapping):
     def __init__(self, view: View) -> None:
         self.view = view
-        self._values: typing.MutableMapping[str, typing.Dict] = {}
+        self._values: MutableMapping[str, dict] = {}
         view.sig_store_remove.connect(self._sig_store_remove)
         view.sig_store_refresh.connect(self._sig_store_refresh)
 
-    def __iter__(self) -> typing.Iterator:
+    def __iter__(self) -> Iterator:
         return iter(self._values)
 
     def __len__(self) -> int:
