@@ -1,8 +1,9 @@
 import uuid
 import warnings
 from abc import ABCMeta
+from collections.abc import Sequence
 from enum import Flag
-from typing import Optional, Sequence, Tuple
+from typing import Literal, Optional
 
 from mitmproxy import certs
 from mitmproxy.coretypes import serializable
@@ -12,16 +13,20 @@ from mitmproxy.utils import human
 
 class ConnectionState(Flag):
     """The current state of the underlying socket."""
+
     CLOSED = 0
     CAN_READ = 1
     CAN_WRITE = 2
     OPEN = CAN_READ | CAN_WRITE
 
 
+TransportProtocol = Literal["tcp", "udp"]
+
+
 # practically speaking we may have IPv6 addresses with flowinfo and scope_id,
 # but type checking isn't good enough to properly handle tuple unions.
 # this version at least provides useful type checking messages.
-Address = Tuple[str, int]
+Address = tuple[str, int]
 
 
 class Connection(serializable.Serializable, metaclass=ABCMeta):
@@ -31,6 +36,7 @@ class Connection(serializable.Serializable, metaclass=ABCMeta):
     The connection object only exposes metadata about the connection, but not the underlying socket object.
     This is intentional, all I/O should be handled by `mitmproxy.proxy.server` exclusively.
     """
+
     # all connections have a unique id. While
     # f.client_conn == f2.client_conn already holds true for live flows (where we have object identity),
     # we also want these semantics for recorded flows.
@@ -38,6 +44,8 @@ class Connection(serializable.Serializable, metaclass=ABCMeta):
     """A unique UUID to identify the connection."""
     state: ConnectionState
     """The current connection state."""
+    transport_protocol: TransportProtocol
+    """The connection protocol in use."""
     peername: Optional[Address]
     """The remote's `(ip, port)` tuple for this connection."""
     sockname: Optional[Address]
@@ -115,25 +123,31 @@ class Connection(serializable.Serializable, metaclass=ABCMeta):
         return hash(self.id)
 
     def __repr__(self):
-        attrs = repr({
-            k: {
-                "cipher_list": lambda: f"<{len(v)} ciphers>",
-                "id": lambda: f"…{v[-6:]}"
-            }.get(k, lambda: v)()
-            for k, v in self.__dict__.items()
-        })
+        attrs = repr(
+            {
+                k: {
+                    "cipher_list": lambda: f"<{len(v)} ciphers>",
+                    "id": lambda: f"…{v[-6:]}",
+                }.get(k, lambda: v)()
+                for k, v in self.__dict__.items()
+            }
+        )
         return f"{type(self).__name__}({attrs})"
 
     @property
     def alpn_proto_negotiated(self) -> Optional[bytes]:  # pragma: no cover
         """*Deprecated:* An outdated alias for Connection.alpn."""
-        warnings.warn("Connection.alpn_proto_negotiated is deprecated, use Connection.alpn instead.",
-                      DeprecationWarning)
+        warnings.warn(
+            "Connection.alpn_proto_negotiated is deprecated, use Connection.alpn instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.alpn
 
 
 class Client(Connection):
     """A connection between a client and mitmproxy."""
+
     peername: Address
     """The client's address."""
     sockname: Address
@@ -147,12 +161,20 @@ class Client(Connection):
     timestamp_start: float
     """*Timestamp:* TCP SYN received"""
 
-    def __init__(self, peername: Address, sockname: Address, timestamp_start: float):
+    def __init__(
+        self,
+        peername: Address,
+        sockname: Address,
+        timestamp_start: float,
+        *,
+        transport_protocol: TransportProtocol = "tcp",
+    ):
         self.id = str(uuid.uuid4())
         self.peername = peername
         self.sockname = sockname
         self.timestamp_start = timestamp_start
         self.state = ConnectionState.OPEN
+        self.transport_protocol = transport_protocol
 
     def __str__(self):
         if self.alpn:
@@ -167,35 +189,33 @@ class Client(Connection):
         # Important: Retain full compatibility with old proxy core for now!
         # This means we need to add all new fields to the old implementation.
         return {
-            'address': self.peername,
-            'alpn': self.alpn,
-            'cipher_name': self.cipher,
-            'id': self.id,
-            'mitmcert': self.mitmcert.get_state() if self.mitmcert is not None else None,
-            'sni': self.sni,
-            'timestamp_end': self.timestamp_end,
-            'timestamp_start': self.timestamp_start,
-            'timestamp_tls_setup': self.timestamp_tls_setup,
-            'tls_established': self.tls_established,
-            'tls_extensions': [],
-            'tls_version': self.tls_version,
+            "address": self.peername,
+            "alpn": self.alpn,
+            "cipher_name": self.cipher,
+            "id": self.id,
+            "mitmcert": self.mitmcert.get_state()
+            if self.mitmcert is not None
+            else None,
+            "sni": self.sni,
+            "timestamp_end": self.timestamp_end,
+            "timestamp_start": self.timestamp_start,
+            "timestamp_tls_setup": self.timestamp_tls_setup,
+            "tls_established": self.tls_established,
+            "tls_extensions": [],
+            "tls_version": self.tls_version,
             # only used in sans-io
-            'state': self.state.value,
-            'sockname': self.sockname,
-            'error': self.error,
-            'tls': self.tls,
-            'certificate_list': [x.get_state() for x in self.certificate_list],
-            'alpn_offers': self.alpn_offers,
-            'cipher_list': self.cipher_list,
+            "state": self.state.value,
+            "sockname": self.sockname,
+            "error": self.error,
+            "tls": self.tls,
+            "certificate_list": [x.get_state() for x in self.certificate_list],
+            "alpn_offers": self.alpn_offers,
+            "cipher_list": self.cipher_list,
         }
 
     @classmethod
     def from_state(cls, state) -> "Client":
-        client = Client(
-            state["address"],
-            ("mitmproxy", 8080),
-            state["timestamp_start"]
-        )
+        client = Client(state["address"], ("mitmproxy", 8080), state["timestamp_start"])
         client.set_state(state)
         return client
 
@@ -214,33 +234,54 @@ class Client(Connection):
         self.sockname = tuple(state["sockname"]) if state["sockname"] else None
         self.error = state["error"]
         self.tls = state["tls"]
-        self.certificate_list = [certs.Cert.from_state(x) for x in state["certificate_list"]]
-        self.mitmcert = certs.Cert.from_state(state["mitmcert"]) if state["mitmcert"] is not None else None
+        self.certificate_list = [
+            certs.Cert.from_state(x) for x in state["certificate_list"]
+        ]
+        self.mitmcert = (
+            certs.Cert.from_state(state["mitmcert"])
+            if state["mitmcert"] is not None
+            else None
+        )
         self.alpn_offers = state["alpn_offers"]
         self.cipher_list = state["cipher_list"]
 
     @property
     def address(self):  # pragma: no cover
         """*Deprecated:* An outdated alias for Client.peername."""
-        warnings.warn("Client.address is deprecated, use Client.peername instead.", DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "Client.address is deprecated, use Client.peername instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.peername
 
     @address.setter
     def address(self, x):  # pragma: no cover
-        warnings.warn("Client.address is deprecated, use Client.peername instead.", DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "Client.address is deprecated, use Client.peername instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         self.peername = x
 
     @property
     def cipher_name(self) -> Optional[str]:  # pragma: no cover
         """*Deprecated:* An outdated alias for Connection.cipher."""
-        warnings.warn("Client.cipher_name is deprecated, use Client.cipher instead.", DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "Client.cipher_name is deprecated, use Client.cipher instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.cipher
 
     @property
     def clientcert(self) -> Optional[certs.Cert]:  # pragma: no cover
         """*Deprecated:* An outdated alias for Connection.certificate_list[0]."""
-        warnings.warn("Client.clientcert is deprecated, use Client.certificate_list instead.", DeprecationWarning,
-                      stacklevel=2)
+        warnings.warn(
+            "Client.clientcert is deprecated, use Client.certificate_list instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         if self.certificate_list:
             return self.certificate_list[0]
         else:
@@ -248,7 +289,11 @@ class Client(Connection):
 
     @clientcert.setter
     def clientcert(self, val):  # pragma: no cover
-        warnings.warn("Client.clientcert is deprecated, use Client.certificate_list instead.", DeprecationWarning)
+        warnings.warn(
+            "Client.clientcert is deprecated, use Client.certificate_list instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         if val:
             self.certificate_list = [val]
         else:
@@ -272,10 +317,16 @@ class Server(Connection):
     via: Optional[server_spec.ServerSpec] = None
     """An optional proxy server specification via which the connection should be established."""
 
-    def __init__(self, address: Optional[Address]):
+    def __init__(
+        self,
+        address: Optional[Address],
+        *,
+        transport_protocol: TransportProtocol = "tcp",
+    ):
         self.id = str(uuid.uuid4())
         self.address = address
         self.state = ConnectionState.CLOSED
+        self.transport_protocol = transport_protocol
 
     def __str__(self):
         if self.alpn:
@@ -291,38 +342,41 @@ class Server(Connection):
         return f"Server({human.format_address(self.address)}, state={self.state.name.lower()}{tls_state}{local_port})"
 
     def __setattr__(self, name, value):
-        if name == "address":
-            connection_open = self.__dict__.get("state", ConnectionState.CLOSED) is ConnectionState.OPEN
+        if name in ("address", "via"):
+            connection_open = (
+                self.__dict__.get("state", ConnectionState.CLOSED)
+                is ConnectionState.OPEN
+            )
             # assigning the current value is okay, that may be an artifact of calling .set_state().
-            address_changed = self.__dict__.get("address") != value
-            if connection_open and address_changed:
-                raise RuntimeError("Cannot change server address on open connection.")
+            attr_changed = self.__dict__.get(name) != value
+            if connection_open and attr_changed:
+                raise RuntimeError(f"Cannot change server.{name} on open connection.")
         return super().__setattr__(name, value)
 
     def get_state(self):
         return {
-            'address': self.address,
-            'alpn': self.alpn,
-            'id': self.id,
-            'ip_address': self.peername,
-            'sni': self.sni,
-            'source_address': self.sockname,
-            'timestamp_end': self.timestamp_end,
-            'timestamp_start': self.timestamp_start,
-            'timestamp_tcp_setup': self.timestamp_tcp_setup,
-            'timestamp_tls_setup': self.timestamp_tls_setup,
-            'tls_established': self.tls_established,
-            'tls_version': self.tls_version,
-            'via': None,
+            "address": self.address,
+            "alpn": self.alpn,
+            "id": self.id,
+            "ip_address": self.peername,
+            "sni": self.sni,
+            "source_address": self.sockname,
+            "timestamp_end": self.timestamp_end,
+            "timestamp_start": self.timestamp_start,
+            "timestamp_tcp_setup": self.timestamp_tcp_setup,
+            "timestamp_tls_setup": self.timestamp_tls_setup,
+            "tls_established": self.tls_established,
+            "tls_version": self.tls_version,
+            "via": None,
             # only used in sans-io
-            'state': self.state.value,
-            'error': self.error,
-            'tls': self.tls,
-            'certificate_list': [x.get_state() for x in self.certificate_list],
-            'alpn_offers': self.alpn_offers,
-            'cipher_name': self.cipher,
-            'cipher_list': self.cipher_list,
-            'via2': self.via,
+            "state": self.state.value,
+            "error": self.error,
+            "tls": self.tls,
+            "certificate_list": [x.get_state() for x in self.certificate_list],
+            "alpn_offers": self.alpn_offers,
+            "cipher_name": self.cipher,
+            "cipher_list": self.cipher_list,
+            "via2": self.via,
         }
 
     @classmethod
@@ -337,7 +391,9 @@ class Server(Connection):
         self.id = state["id"]
         self.peername = tuple(state["ip_address"]) if state["ip_address"] else None
         self.sni = state["sni"]
-        self.sockname = tuple(state["source_address"]) if state["source_address"] else None
+        self.sockname = (
+            tuple(state["source_address"]) if state["source_address"] else None
+        )
         self.timestamp_end = state["timestamp_end"]
         self.timestamp_start = state["timestamp_start"]
         self.timestamp_tcp_setup = state["timestamp_tcp_setup"]
@@ -346,7 +402,9 @@ class Server(Connection):
         self.state = ConnectionState(state["state"])
         self.error = state["error"]
         self.tls = state["tls"]
-        self.certificate_list = [certs.Cert.from_state(x) for x in state["certificate_list"]]
+        self.certificate_list = [
+            certs.Cert.from_state(x) for x in state["certificate_list"]
+        ]
         self.alpn_offers = state["alpn_offers"]
         self.cipher = state["cipher_name"]
         self.cipher_list = state["cipher_list"]
@@ -355,14 +413,21 @@ class Server(Connection):
     @property
     def ip_address(self) -> Optional[Address]:  # pragma: no cover
         """*Deprecated:* An outdated alias for `Server.peername`."""
-        warnings.warn("Server.ip_address is deprecated, use Server.peername instead.", DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "Server.ip_address is deprecated, use Server.peername instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.peername
 
     @property
     def cert(self) -> Optional[certs.Cert]:  # pragma: no cover
         """*Deprecated:* An outdated alias for `Connection.certificate_list[0]`."""
-        warnings.warn("Server.cert is deprecated, use Server.certificate_list instead.", DeprecationWarning,
-                      stacklevel=2)
+        warnings.warn(
+            "Server.cert is deprecated, use Server.certificate_list instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         if self.certificate_list:
             return self.certificate_list[0]
         else:
@@ -370,17 +435,15 @@ class Server(Connection):
 
     @cert.setter
     def cert(self, val):  # pragma: no cover
-        warnings.warn("Server.cert is deprecated, use Server.certificate_list instead.", DeprecationWarning,
-                      stacklevel=2)
+        warnings.warn(
+            "Server.cert is deprecated, use Server.certificate_list instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         if val:
             self.certificate_list = [val]
         else:
             self.certificate_list = []
 
 
-__all__ = [
-    "Connection",
-    "Client",
-    "Server",
-    "ConnectionState"
-]
+__all__ = ["Connection", "Client", "Server", "ConnectionState"]
